@@ -41,6 +41,10 @@ def switch(channel):
     run(["pijul", "channel", "switch", channel], check=True, stdout=DEVNULL)
 
 
+def delete(channel):
+    run(["pijul", "channel", "delete", channel], check=True, stdout=DEVNULL)
+
+
 def ancestry_path(head, base):
     res = run(
         ["git", "rev-list", "--ancestry-path", f"{base}..{head}"],
@@ -95,6 +99,23 @@ def record(log, author, timestamp):
     return res.stdout.strip().decode("UTF-8"), res.stderr.strip().decode("UTF-8")
 
 
+def record_simple(log):
+    res = run(
+        [
+            "pijul",
+            "record",
+            "--all",
+            "--message",
+            log,
+        ],
+        check=True,
+        env=batch,
+        stdout=PIPE,
+        stderr=PIPE,
+    )
+    return res.stdout.strip().decode("UTF-8"), res.stderr.strip().decode("UTF-8")
+
+
 def get_base(head):
     return (
         run(
@@ -177,6 +198,7 @@ class Runner:
     def __init__(self, revs):
         self.revs = revs
         self.here = Path(".").absolute()
+        self.error = False
 
     def run(self):
         prev = [self.revs.pop()]
@@ -191,7 +213,7 @@ class Runner:
                     rev = self.revs.pop()
                     pbar.update()
                 self.step(prev[-1], rev)
-            finally:
+            except:  # noqa
                 info = f"commit {change()}"
                 prev.append(rev)
                 for last in reversed(prev):
@@ -199,7 +221,9 @@ class Runner:
                         fork(last)
                         pijul_restore()
                         switch(last)
+                        self.error = True
                         break
+                raise
 
     def prepare(self, prev, rev):
         checkout(rev)
@@ -257,16 +281,52 @@ def prepare_workdir(workdir, tmp_dir):
     Path(".pijul").symlink_to(Path(workdir, ".pijul"))
 
 
+def add_ignore():
+    ignore = Path(".ignore").absolute()
+    if not ignore.exists():
+        with open(bytes(ignore), "w") as f:
+            f.truncate(0)
+            f.write(".git\n")
+
+
+def run_it(head, base):
+    add_ignore()
+    revs = rev_list(head, base)
+    runner = Runner(revs)
+    try:
+        runner.run()
+    except:  # noqa
+        if runner.error:
+            delete(head)
+        raise
+
+
 @click.group()
 def main():
     pass
 
 
+@main.command()
+def shallow():
+    """create a new pijul repository from current revision without history"""
+    workdir = Path(".").absolute()
+    check_git()
+    check_init()
+    with TemporaryDirectory() as tmp_dir:
+        prepare_workdir(workdir, tmp_dir)
+        head, _ = get_head()
+        fork(head)
+        switch(head)
+        add_ignore()
+        add_recursive()
+        record_simple(f"commit {head}")
+
+
 @main.command()
 @click.option("--base", default=None, help="Update from (commit-ish, default '--root')")
 @click.option("--head", default=None, help="Update to (commit-ish, default HEAD)")
 def update(base, head):
-    """update a repository create with git-pijul"""
+    """update a repository created with git-pijul"""
     workdir = Path(".").absolute()
     check_git()
     if not Path(".pijul").exists():
@@ -286,10 +346,10 @@ def update(base, head):
         prepare_workdir(workdir, tmp_dir)
         pijul_restore()
         switch(base)
+        fork(head)
+        switch(head)
         git_restore()
-        revs = rev_list(head, base)
-        runner = Runner(revs)
-        runner.run()
+        run_it(head, base)
 
 
 @main.command()
@@ -306,12 +366,9 @@ def create(base, head):
         print(f"Using base: {base} ('--root')")
     with TemporaryDirectory() as tmp_dir:
         prepare_workdir(workdir, tmp_dir)
-        with open(".ignore", "w") as f:
-            f.truncate(0)
-            f.write(".git\n")
-        revs = rev_list(head, base)
-        runner = Runner(revs)
-        runner.run()
+        fork(head)
+        switch(head)
+        run_it(head, base)
 
 
 if __name__ == "__main__":
